#!/bin/bash

ret=0

setup_rt_networking()
{
	local area=$1
        local rt=$2
        local rt2=$3
        local nsname=n${rt}

        ip netns add ${nsname}
        ip link set veth-rt-${rt} name veth0
        ip link set veth-rt-${rt2} name veth1
        ip link set veth0 netns ${nsname}
        ip link set veth1 netns ${nsname}

        ip netns exec ${nsname} sysctl -wq net.ipv6.conf.all.forwarding=1

	# setup some routes
	ip -netns ${nsname} addr add fdfd:${area}:${rt}::1/56 dev lo
	ip -netns ${nsname} addr add fdfd:${area}:${rt}:1::${rt}/64 dev veth0
	ip -netns ${nsname} addr add fdfd:${area}:${rt}:2::${rt}/64 dev veth1
        
	ip -netns ${nsname} link set veth0 up
        ip -netns ${nsname} link set veth1 up
        ip -netns ${nsname} link set lo up

	ARGS="--config=n$rt.conf"
	ARGS="$ARGS --v=3"
	ip netns exec ${nsname} run_openr.sh "" "$ARGS" > openr-${nsname}.log 2>&1 &
}

setup()
{
	# setup virtual interface pairs
	ip link add veth-rt-1 type veth peer name veth-rt-2
	ip link add veth-rt-3 type veth peer name veth-rt-4
	ip link add veth-rt-5 type veth peer name veth-rt-6
	ip link add veth-rt-7 type veth peer name veth-rt-8

	# N1 -veth0 A1 veth0- N2 - veth1 A2 veth1 N3
	# N1 - veth1 X veth0 N3
	setup_rt_networking 1 1 7 
	setup_rt_networking 2 2 4
	setup_rt_networking 2 3 5 
}

cleanup()
{
	killall run_openr.sh
	killall openr

        ip link del veth-rt-1 2>/dev/null || true
        ip link del veth-rt-2 2>/dev/null || true
        ip link del veth-rt-3 2>/dev/null || true
        ip link del veth-rt-4 2>/dev/null || true
        ip link del veth-rt-5 2>/dev/null || true
        ip link del veth-rt-6 2>/dev/null || true
        ip link del veth-rt-7 2>/dev/null || true
        ip link del veth-rt-8 2>/dev/null || true

        # destroy namespaces
        for ns in $(ip netns show | grep -E 'n*'); do
                ip netns del ${ns} || true
        done
}

cleanup &>/dev/null

[ "$1" = "stop" ] && exit 0

set -x
setup

exit ${ret}
